@@ -31,6 +31,18 @@
 #endif
 
 
+
+
+
+
+
+
+
+
+
+
+
+
 #include <errno.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -125,6 +137,12 @@ int mutt_display_message (HEADER *cur)
 
 
 
+
+
+
+
+
+
   mutt_mktemp (tempfile);
   if ((fpout = safe_fopen (tempfile, "w")) == NULL)
   {
@@ -136,7 +154,7 @@ int mutt_display_message (HEADER *cur)
     builtin = 1;
   else
   {
-    mutt_make_string (buf, sizeof (buf), PagerFmt, cur);
+    mutt_make_string (buf, sizeof (buf), PagerFmt, Context, cur);
     fputs (buf, fpout);
     fputs ("\n\n", fpout);
   }
@@ -193,7 +211,7 @@ void ci_bounce_message (HEADER *h, int *redraw)
   ADDRESS *adr = NULL;
   int rc;
 
-  snprintf (prompt, sizeof (prompt), "Bounce %smessage%s to: ",
+  snprintf (prompt, sizeof(prompt), "Bounce %smessage%s to: ",
 	    h ? "" : "tagged ", h ? "" : "s");
   rc = mutt_get_field (prompt, buf, sizeof (buf), M_ALIAS);
 
@@ -214,8 +232,12 @@ void ci_bounce_message (HEADER *h, int *redraw)
 
   adr = mutt_expand_aliases (adr);
 
-  snprintf (prompt, sizeof (prompt), "Bounce message%s to %s...?",
-	    (h ? "" : "s"), adr->mailbox);
+  buf[0] = 0;
+  rfc822_write_address (buf, sizeof (buf), adr);
+
+  snprintf (prompt, (COLS > sizeof(prompt) ? sizeof(prompt) : COLS) - 13, 
+            "Bounce message%s to %s", (h ? "" : "s"), buf);
+  strcat(prompt, "...?");
   if (mutt_yesorno (prompt, 1) != 1)
   {
     rfc822_free_address (&adr);
@@ -255,6 +277,12 @@ int mutt_pipe_message (HEADER *h)
   endwin ();
   if (h)
   {
+
+
+
+
+
+
     thepid = mutt_create_filter (buffer, &s.fpout, NULL, NULL);
     mutt_pipe_message_to_state (h, &s);
     fclose (s.fpout);
@@ -262,7 +290,12 @@ int mutt_pipe_message (HEADER *h)
   }
   else
   { /* handle tagged messages */
+
+
+
     
+
+
     if (option (OPTPIPESPLIT))
     {
       for (i = 0; i < Context->vcount; i++)
@@ -273,7 +306,7 @@ int mutt_pipe_message (HEADER *h)
 	  thepid = mutt_create_filter (buffer, &(s.fpout), NULL, NULL);
           mutt_pipe_message_to_state (Context->hdrs[Context->v2r[i]], &s);
           /* add the message separator */
-          if (PipeSep[0])
+          if (PipeSep)
 	    state_puts (PipeSep, &s);
 	  fclose (s.fpout);
 	  if (mutt_wait_filter (thepid) != 0)
@@ -291,7 +324,7 @@ int mutt_pipe_message (HEADER *h)
         {
           mutt_pipe_message_to_state (Context->hdrs[Context->v2r[i]], &s);
           /* add the message separator */
-          if (PipeSep[0])
+          if (PipeSep)
 	    state_puts (PipeSep, &s);
         }
       }
@@ -386,22 +419,25 @@ void mutt_shell_escape (void)
 /* enter a mutt command */
 void mutt_enter_command (void)
 {
+  BUFFER err, token;
   char buffer[LONG_STRING], errbuf[SHORT_STRING];
   int r;
   int old_strictthreads = option (OPTSTRICTTHREADS);
   int old_sortre	= option (OPTSORTRE);
 
   buffer[0] = 0;
-  if (mutt_get_field (":", buffer, sizeof (buffer), 0) != 0 || !buffer[0])
+  if (mutt_get_field (":", buffer, sizeof (buffer), M_COMMAND) != 0 || !buffer[0])
     return;
-  r = mutt_parse_rc_line (buffer, errbuf, sizeof (errbuf));
+  err.data = errbuf;
+  err.dsize = sizeof (errbuf);
+  memset (&token, 0, sizeof (token));
+  r = mutt_parse_rc_line (buffer, &token, &err);
+  FREE (&token.data);
   if (errbuf[0])
   {
-    /*
-     * since errbuf could potentially contain printf() sequences in it,
-     * we must call mutt_error() in this fashion so that vsprintf()
-     * doesn't expect more arguments that we passed
-     */
+    /* since errbuf could potentially contain printf() sequences in it,
+       we must call mutt_error() in this fashion so that vsprintf()
+       doesn't expect more arguments that we passed */
     if (r == 0)
       mutt_message ("%s", errbuf);
     else
@@ -426,10 +462,13 @@ static int save_confirm_func (const char *s, struct stat *st)
 {
   char tmp[_POSIX_PATH_MAX];
   int ret = 1;
+  int magic = 0;
+
+  magic = mx_get_magic (s);
 
   if (stat (s, st) != -1)
   {
-    if (mx_get_magic (s) == -1)
+    if (magic == -1)
     {
       mutt_error ("%s is not a mailbox!", s);
       return 0;
@@ -444,22 +483,25 @@ static int save_confirm_func (const char *s, struct stat *st)
   }
   else
   {
-    st->st_mtime = 0;
-    st->st_atime = 0;
-    
-    if (errno == ENOENT)
+    if (magic != M_IMAP)
     {
-      if (option (OPTCONFIRMCREATE))
+      st->st_mtime = 0;
+      st->st_atime = 0;
+
+      if (errno == ENOENT)
       {
-	snprintf (tmp, sizeof (tmp), "Create %s?", s);
-	if (mutt_yesorno (tmp, 1) < 1)
-	  ret = 0;
+	if (option (OPTCONFIRMCREATE))
+	{
+	  snprintf (tmp, sizeof (tmp), "Create %s?", s);
+	  if (mutt_yesorno (tmp, 1) < 1)
+	    ret = 0;
+	}
+      }
+      else
+      {
+	mutt_perror (s);
+	return 0;
       }
-    }
-    else
-    {
-      mutt_perror (s);
-      return 0;
     }
   }
 
