@@ -37,30 +37,33 @@ struct mapping_t Menus[] = {
  { "generic",	MENU_GENERIC },
  { "index",	MENU_MAIN },
  { "pager",	MENU_PAGER },
+ { "postpone",	MENU_POST },
 
   
 
   
   
+
+ { "query",	MENU_QUERY },
  { NULL,	0 }
 };
 
 #define mutt_check_menu(s) mutt_getvaluebyname(s, Menus)
 
 static struct mapping_t KeyNames[] = {
-  { "pageup",	KEY_PPAGE },
-  { "pagedown",	KEY_NPAGE },
-  { "up",	KEY_UP },
-  { "down",	KEY_DOWN },
-  { "right",	KEY_RIGHT },
-  { "left",	KEY_LEFT },
-  { "delete",	KEY_DC },
-  { "backspace",KEY_BACKSPACE },
-  { "insert",	KEY_IC },
-  { "home",	KEY_HOME },
-  { "end",	KEY_END },
-  { "enter",	KEY_ENTER },
-  { "return",	M_ENTER_C },
+  { "<PageUp>",	KEY_PPAGE },
+  { "<PageDown>",	KEY_NPAGE },
+  { "<Up>",	KEY_UP },
+  { "<Down>",	KEY_DOWN },
+  { "<Right>",	KEY_RIGHT },
+  { "<Left>",	KEY_LEFT },
+  { "<Delete>",	KEY_DC },
+  { "<BackSpace>",KEY_BACKSPACE },
+  { "<Insert>",	KEY_IC },
+  { "<Home>",	KEY_HOME },
+  { "<End>",	KEY_END },
+  { "<Enter>",	KEY_ENTER },
+  { "<Return>",	M_ENTER_C },
   { NULL,	0 }
 };
 
@@ -91,10 +94,10 @@ static int parsekeys (char *s, keycode_t *d, int max)
       s += strlen (s);
       *d = n;
     }
-    else if (tolower (*s) == 'f' && isdigit (*(s + 1)))
+    else if (tolower (*s) == 'f' && isdigit ((unsigned char) s[1]))
     {
       n = 0;
-      for (s++ ; isdigit (*s) ; s++)
+      for (s++; isdigit ((unsigned char) *s) ; s++)
       {
 	n *= 10;
 	n += *s - '0';
@@ -179,10 +182,37 @@ void km_bindkey (char *s, int menu, int op, char *macro)
 
 static void push_string (char *s)
 {
-  char *p = s + strlen (s) - 1;
+  char *pp, *p = s + strlen (s) - 1;
+  size_t l;
+  int i;
 
   while (p >= s)
+  {
+    /* if we see something like "<PageUp>", look to see if it is a real
+       function name and return the corresponding value */
+    if (*p == '>')
+    {
+      for (pp = p - 1; pp >= s && *pp != '<'; pp--)
+	;
+      if (pp >= s)
+      {
+	l = p - pp + 1;
+	for (i = 0; KeyNames[i].name; i++)
+	{
+	  if (!strncasecmp (pp, KeyNames[i].name, l))
+	    break;
+	}
+	if (KeyNames[i].name)
+	{
+	  /* found a match */
+	  mutt_ungetch (KeyNames[i].value);
+	  p = pp - 1;
+	  continue;
+	}
+      }
+    }
     mutt_ungetch (*p--);
+  }
 }
 
 static int retry_generic (int menu, keycode_t *keys, int keyslen, int lastkey)
@@ -265,6 +295,10 @@ static void create_bindings (struct binding_t *map, int menu)
 char *km_keyname (int c)
 {
   static char buf[5];
+  char *p;
+
+  if ((p = mutt_getnamebyvalue (c, KeyNames)))
+    return p;
 
   switch (c)
   {
@@ -277,28 +311,6 @@ char *km_keyname (int c)
       return "RET";
     case '\t':
       return "TAB";
-    case KEY_UP:
-      return "up";
-    case KEY_DOWN:
-      return "down";
-    case KEY_LEFT:
-      return "left";
-    case KEY_RIGHT:
-      return "right";
-    case KEY_NPAGE:
-      return "pagedown";
-    case KEY_PPAGE:
-      return "pageup";
-    case KEY_BACKSPACE:
-      return "backspace";
-    case KEY_HOME:
-      return "home";
-    case KEY_END:
-      return "end";
-    case KEY_DC:
-      return "DEL";
-    case KEY_IC:
-      return "INS";
   }
 
   if (c < 256 && c > -128 && iscntrl ((unsigned char) c))
@@ -367,31 +379,37 @@ void km_init (void)
   create_bindings (OpMain, MENU_MAIN);
   create_bindings (OpPager, MENU_PAGER);
   create_bindings (OpPost, MENU_POST);
+  create_bindings (OpQuery, MENU_QUERY);
 
+
+
+
+
+  
   /* bindings for the line editor */
   create_bindings (OpEditor, MENU_EDITOR);
-
-  km_bindkey ("up", MENU_EDITOR, OP_EDITOR_HISTORY_UP, NULL);
-  km_bindkey ("down", MENU_EDITOR, OP_EDITOR_HISTORY_DOWN, NULL);
-  km_bindkey ("left", MENU_EDITOR, OP_EDITOR_BACKWARD_CHAR, NULL);
-  km_bindkey ("right", MENU_EDITOR, OP_EDITOR_FORWARD_CHAR, NULL);
-  km_bindkey ("home", MENU_EDITOR, OP_EDITOR_BOL, NULL);
-  km_bindkey ("end", MENU_EDITOR, OP_EDITOR_EOL, NULL);
-  km_bindkey ("backspace", MENU_EDITOR, OP_EDITOR_BACKSPACE, NULL);
-  km_bindkey ("delete", MENU_EDITOR, OP_EDITOR_BACKSPACE, NULL);
+  
+  km_bindkey ("<up>", MENU_EDITOR, OP_EDITOR_HISTORY_UP, NULL);
+  km_bindkey ("<down>", MENU_EDITOR, OP_EDITOR_HISTORY_DOWN, NULL);
+  km_bindkey ("<left>", MENU_EDITOR, OP_EDITOR_BACKWARD_CHAR, NULL);
+  km_bindkey ("<right>", MENU_EDITOR, OP_EDITOR_FORWARD_CHAR, NULL);
+  km_bindkey ("<home>", MENU_EDITOR, OP_EDITOR_BOL, NULL);
+  km_bindkey ("<end>", MENU_EDITOR, OP_EDITOR_EOL, NULL);
+  km_bindkey ("<backspace>", MENU_EDITOR, OP_EDITOR_BACKSPACE, NULL);
+  km_bindkey ("<delete>", MENU_EDITOR, OP_EDITOR_BACKSPACE, NULL);
   km_bindkey ("\177", MENU_EDITOR, OP_EDITOR_BACKSPACE, NULL);
-
+  
   /* generic menu keymap */
   create_bindings (OpGeneric, MENU_GENERIC);
-
-  km_bindkey ("home", MENU_GENERIC, OP_FIRST_ENTRY, NULL);
-  km_bindkey ("end", MENU_GENERIC, OP_LAST_ENTRY, NULL);
-  km_bindkey ("pagedown", MENU_GENERIC, OP_NEXT_PAGE, NULL);
-  km_bindkey ("pageup", MENU_GENERIC, OP_PREV_PAGE, NULL);
-  km_bindkey ("right", MENU_GENERIC, OP_NEXT_PAGE, NULL);
-  km_bindkey ("left", MENU_GENERIC, OP_PREV_PAGE, NULL);
-  km_bindkey ("up", MENU_GENERIC, OP_PREV_ENTRY, NULL);
-  km_bindkey ("down", MENU_GENERIC, OP_NEXT_ENTRY, NULL);
+  
+  km_bindkey ("<home>", MENU_GENERIC, OP_FIRST_ENTRY, NULL);
+  km_bindkey ("<end>", MENU_GENERIC, OP_LAST_ENTRY, NULL);
+  km_bindkey ("<pagedown>", MENU_GENERIC, OP_NEXT_PAGE, NULL);
+  km_bindkey ("<pageup>", MENU_GENERIC, OP_PREV_PAGE, NULL);
+  km_bindkey ("<right>", MENU_GENERIC, OP_NEXT_PAGE, NULL);
+  km_bindkey ("<left>", MENU_GENERIC, OP_PREV_PAGE, NULL);
+  km_bindkey ("<up>", MENU_GENERIC, OP_PREV_ENTRY, NULL);
+  km_bindkey ("<down>", MENU_GENERIC, OP_NEXT_ENTRY, NULL);
   km_bindkey ("1", MENU_GENERIC, OP_JUMP, NULL);
   km_bindkey ("2", MENU_GENERIC, OP_JUMP, NULL);
   km_bindkey ("3", MENU_GENERIC, OP_JUMP, NULL);
@@ -403,25 +421,25 @@ void km_init (void)
   km_bindkey ("9", MENU_GENERIC, OP_JUMP, NULL);
 
   /* Miscellaneous extra bindings */
-
+  
   km_bindkey (" ", MENU_MAIN, OP_DISPLAY_MESSAGE, NULL);
-  km_bindkey ("up", MENU_MAIN, OP_MAIN_PREV_UNDELETED, NULL);
-  km_bindkey ("down", MENU_MAIN, OP_MAIN_NEXT_UNDELETED, NULL);
+  km_bindkey ("<up>", MENU_MAIN, OP_MAIN_PREV_UNDELETED, NULL);
+  km_bindkey ("<down>", MENU_MAIN, OP_MAIN_NEXT_UNDELETED, NULL);
   km_bindkey ("J", MENU_MAIN, OP_NEXT_ENTRY, NULL);
   km_bindkey ("K", MENU_MAIN, OP_PREV_ENTRY, NULL);
   km_bindkey ("x", MENU_MAIN, OP_EXIT, NULL);
 
   km_bindkey ("x", MENU_PAGER, OP_PAGER_EXIT, NULL);
   km_bindkey ("q", MENU_PAGER, OP_PAGER_EXIT, NULL);
-  km_bindkey ("backspace", MENU_PAGER, OP_PREV_LINE, NULL);
-  km_bindkey ("pagedown", MENU_PAGER, OP_NEXT_PAGE, NULL);
-  km_bindkey ("pageup", MENU_PAGER, OP_PREV_PAGE, NULL);
-  km_bindkey ("up", MENU_PAGER, OP_MAIN_PREV_UNDELETED, NULL);
-  km_bindkey ("right", MENU_PAGER, OP_MAIN_NEXT_UNDELETED, NULL);
-  km_bindkey ("down", MENU_PAGER, OP_MAIN_NEXT_UNDELETED, NULL);
-  km_bindkey ("left", MENU_PAGER, OP_MAIN_PREV_UNDELETED, NULL);
-  km_bindkey ("home", MENU_PAGER, OP_PAGER_TOP, NULL);
-  km_bindkey ("end", MENU_PAGER, OP_PAGER_BOTTOM, NULL);
+  km_bindkey ("<backspace>", MENU_PAGER, OP_PREV_LINE, NULL);
+  km_bindkey ("<pagedown>", MENU_PAGER, OP_NEXT_PAGE, NULL);
+  km_bindkey ("<pageup>", MENU_PAGER, OP_PREV_PAGE, NULL);
+  km_bindkey ("<up>", MENU_PAGER, OP_MAIN_PREV_UNDELETED, NULL);
+  km_bindkey ("<right>", MENU_PAGER, OP_MAIN_NEXT_UNDELETED, NULL);
+  km_bindkey ("<down>", MENU_PAGER, OP_MAIN_NEXT_UNDELETED, NULL);
+  km_bindkey ("<left>", MENU_PAGER, OP_MAIN_PREV_UNDELETED, NULL);
+  km_bindkey ("<home>", MENU_PAGER, OP_PAGER_TOP, NULL);
+  km_bindkey ("<end>", MENU_PAGER, OP_PAGER_BOTTOM, NULL);
   km_bindkey ("1", MENU_PAGER, OP_JUMP, NULL);
   km_bindkey ("2", MENU_PAGER, OP_JUMP, NULL);
   km_bindkey ("3", MENU_PAGER, OP_JUMP, NULL);
@@ -432,7 +450,7 @@ void km_init (void)
   km_bindkey ("8", MENU_PAGER, OP_JUMP, NULL);
   km_bindkey ("9", MENU_PAGER, OP_JUMP, NULL);
 
-  km_bindkey ("return", MENU_ALIAS, OP_TAG, NULL);
+  km_bindkey ("<return>", MENU_ALIAS, OP_TAG, NULL);
 }
 
 void km_error_key (int menu)
@@ -445,58 +463,54 @@ void km_error_key (int menu)
     mutt_error ("Key is not bound.  See the manual.");
 }
 
-int mutt_parse_push (const char *s, unsigned long data, char *err, size_t errlen)
+int mutt_parse_push (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  char buf[SHORT_STRING];
-  char expn[SHORT_STRING];
+  int r = 0;
 
-  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), M_CONDENSE);
-  if (s)
+  mutt_extract_token (buf, s, M_TOKEN_CONDENSE);
+  if (MoreArgs (s))
   {
-    strfcpy (err, "too many arguments", errlen);
-    return (-1);
+    strfcpy (err->data, "push: too many arguments", err->dsize);
+    r = -1;
   }
-
-  push_string (buf);
-  return (0);
+  else
+    push_string (buf->data);
+  return (r);
 }
 
 /* expects to see: <menu-string> <key-string> */
-static const char *parse_keymap (int *menu,
-				 char *key,
-				 size_t keylen,
-				 const char *s,
-				 char *err,
-				 size_t errlen)
+char *parse_keymap (int *menu, BUFFER *s, BUFFER *err)
 {
-  char buf[SHORT_STRING];
-  char expn[SHORT_STRING];
+  BUFFER buf;
+
+  memset (&buf, 0, sizeof (buf));
 
   /* menu name */
-  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
-
-  if (s)
+  mutt_extract_token (&buf, s, 0);
+  if (MoreArgs (s))
   {
-    if ((*menu = mutt_check_menu (buf)) == -1)
+    if ((*menu = mutt_check_menu (buf.data)) == -1)
     {
-      snprintf (err, errlen, "%s: no such menu", s);
-      return (NULL);
+      snprintf (err->data, err->dsize, "%s: no such menu", buf.data);
     }
-
-    /* key sequence */
-    key[0] = 0;
-    s = mutt_extract_token (key, keylen, s, expn, sizeof (expn), 0);
-    if (!key[0])
+    else
     {
-      strfcpy (err, "null key sequence", errlen);
-      return (NULL);
-    }
+      /* key sequence */
+      mutt_extract_token (&buf, s, 0);
 
-    if (s)
-      return s;
+      if (!*buf.data)
+      {
+	strfcpy (err->data, "null key sequence", err->dsize);
+      }
+      else if (MoreArgs (s))
+	return (buf.data);
+    }
   }
-
-  strfcpy (err, "too few arguments", errlen);
+  else
+  {
+    strfcpy (err->data, "too few arguments", err->dsize);
+  }
+  FREE (&buf.data);
   return (NULL);
 }
 
@@ -534,79 +548,80 @@ struct binding_t *km_get_table (int menu)
       return OpAttach;
     case MENU_EDITOR:
       return OpEditor;
+    case MENU_QUERY:
+      return OpQuery;
+
+
+
+
+
+
   }
   return NULL;
 }
 
 /* bind menu-name '<key_sequence>' function-name */
-int mutt_parse_bind (const char *s, unsigned long data, char *err, size_t errlen)
+int mutt_parse_bind (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
   struct binding_t *bindings = NULL;
-  char buf[SHORT_STRING];
-  char key[SHORT_STRING];
-  char expn[SHORT_STRING];
-  int menu;
+  char *key;
+  int menu, r = 0;
 
-  if ((s = parse_keymap (&menu, key, sizeof (key), s, err, errlen)) == NULL)
+  if ((key = parse_keymap (&menu, s, err)) == NULL)
     return (-1);
 
   /* function to execute */
-  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), 0);
-  if (s)
+  mutt_extract_token (buf, s, 0);
+  if (MoreArgs (s))
   {
-    strfcpy (err, "too many arguments", errlen);
-    return (-1);
+    strfcpy (err->data, "bind: too many arguments", err->dsize);
+    r = -1;
   }
-
-  if (strcasecmp ("noop", buf) == 0)
+  else if (strcasecmp ("noop", buf->data) == 0)
+    km_bindkey (key, menu, OP_NULL, NULL); /* the `unbind' command */
+  else
   {
-    km_bindkey (key, menu, OP_NULL, NULL);
-    return 0;
+    /* First check the "generic" list of commands */
+    if (menu == MENU_PAGER || menu == MENU_EDITOR || menu == MENU_GENERIC ||
+	try_bind (key, menu, buf->data, OpGeneric) != 0)
+    {
+      /* Now check the menu-specific list of commands (if they exist) */
+      bindings = km_get_table (menu);
+      if (bindings && try_bind (key, menu, buf->data, bindings) != 0)
+      {
+	snprintf (err->data, err->dsize, "%s: no such function in map", buf->data);
+	r = -1;
+      }
+    }
   }
-
-  if (menu != MENU_PAGER && menu != MENU_EDITOR && menu != MENU_GENERIC)
-  {
-    /* First check the "generic" list of commands.  */
-    if (try_bind (key, menu, buf, OpGeneric) == 0)
-      return 0;
-  }
-
-  /* Now check the menu-specific list of commands (if they exist).  */
-  bindings = km_get_table (menu);
-  if (bindings && try_bind (key, menu, buf, bindings) == 0)
-    return 0;
-
-  snprintf (err, errlen, "%s: no such function in map", buf);
-  return (-1);
+  FREE (&key);
+  return (r);
 }
 
 /* macro <menu> <key> <macro> */
-int
-mutt_parse_macro (const char *s, unsigned long data, char *err, size_t errlen)
+int mutt_parse_macro (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
 {
-  int menu;
-  char key[SHORT_STRING];
-  char buf[SHORT_STRING];
-  char expn[SHORT_STRING];
+  int menu, r = -1;
+  char *key;
 
-  if ((s = parse_keymap (&menu, key, sizeof (key), s, err, errlen)) == NULL)
+  if ((key = parse_keymap (&menu, s, err)) == NULL)
     return (-1);
 
-  buf[0] = 0;
-  s = mutt_extract_token (buf, sizeof (buf), s, expn, sizeof (expn), M_CONDENSE);
+  mutt_extract_token (buf, s, M_TOKEN_CONDENSE);
   /* make sure the macro sequence is not an empty string */
-  if (!buf[0])
+  if (!*buf->data)
   {
-    strfcpy (err, "empty macro sequence", errlen);
-    return (-1);
+    strfcpy (err->data, "macro: empty key sequence", err->dsize);
   }
-  if (s)
+  else if (MoreArgs (s))
   {
-    strfcpy (err, "too many arguments", errlen);
-    return (-1);
+    strfcpy (err->data, "macro: too many arguments", err->dsize);
   }
-
-  km_bindkey (key, menu, OP_MACRO, buf);
-
-  return 0;
+  else
+  {
+    km_bindkey (key, menu, OP_MACRO, buf->data);
+    r = 0;
+  }
+  FREE (&key);
+  return (r);
 }
