@@ -22,8 +22,14 @@
 #include "sort.h"
 #include "mailbox.h"
 #include "copy.h"
+#include "keymap.h"
 
 
+
+#ifdef USE_IMAP
+#include "imap.h"
+#endif
+
 #ifdef BUFFY_SIZE
 #include "buffy.h"
 #endif
@@ -358,6 +364,16 @@ FILE *mx_open_file_lock (const char *path, const char *mode)
  *	0	not a mailbox
  *	-1	error
  */
+
+#ifdef USE_IMAP
+
+static int mx_is_imap(const char *p)
+{
+  return p && (*p == '{');
+}
+
+#endif
+
 int mx_get_magic (const char *path)
 {
   struct stat st;
@@ -366,7 +382,7 @@ int mx_get_magic (const char *path)
   FILE *f;
 
 #ifdef USE_IMAP
-  if (*path == '{')
+  if(mx_is_imap(path))
     return M_IMAP;
 #endif /* USE_IMAP */
 
@@ -458,10 +474,22 @@ int mx_set_magic (const char *s)
 
 static int mx_open_mailbox_append (CONTEXT *ctx)
 {
+  struct stat sb;
+
   ctx->append = 1;
-  if (access (ctx->path, W_OK) == 0)
+
+#ifdef USE_IMAP
+  
+  if(mx_is_imap(ctx->path))  
+    return imap_open_mailbox_append (ctx);
+
+#endif
+  
+  if(stat(ctx->path, &sb) == 0)
   {
-    switch (ctx->magic = mx_get_magic (ctx->path))
+    ctx->magic = mx_get_magic (ctx->path);
+    
+    switch (ctx->magic)
     {
       case 0:
 	mutt_error ("%s is not a mailbox.", ctx->path);
@@ -696,6 +724,9 @@ void mx_fastclose_mailbox (CONTEXT *ctx)
   safe_free ((void **) &ctx->hdrs);
   safe_free ((void **) &ctx->v2r);
   safe_free ((void **) &ctx->path);
+  safe_free ((void **) &ctx->pattern);
+  if (ctx->limit_pattern) 
+    mutt_pattern_free (&ctx->limit_pattern);
   if (ctx->fp)
     fclose (ctx->fp);
   memset (ctx, 0, sizeof (CONTEXT));
@@ -798,7 +829,9 @@ int mx_close_mailbox (CONTEXT *ctx)
 
   if (ctx->deleted)
   {
-    if ((purge = query_quadoption (OPT_DELETE, "Purge deleted messages?")) < 0)
+    snprintf (buf, sizeof (buf), "Purge %d deleted message%s?",
+	      ctx->deleted, ctx->deleted == 1 ? "" : "s");
+    if ((purge = query_quadoption (OPT_DELETE, buf)) < 0)
       return (-1);
   }
 
@@ -879,7 +912,14 @@ int mx_sync_mailbox (CONTEXT *ctx)
 
   if (ctx->dontwrite)
   {
-    mutt_error ("Mailbox is marked unwritable.");
+    char buf[STRING], tmp[STRING];
+    if (km_expand_key (buf, sizeof(buf),
+                       km_find_func (MENU_MAIN, OP_TOGGLE_WRITE)))
+      snprintf (tmp, sizeof(tmp), " Press '%s' to toggle write", buf);
+    else
+      strfcpy (tmp, "Use 'toggle-write' to re-enable write!", sizeof(tmp));
+
+    mutt_error ("Mailbox is marked unwritable. %s", tmp);
     return -1;
   }
   else if (ctx->readonly)
@@ -896,7 +936,11 @@ int mx_sync_mailbox (CONTEXT *ctx)
 
   if (ctx->deleted)
   {
-    if ((rc = query_quadoption (OPT_DELETE, "Purge deleted messages?")) < 0)
+    char buf[SHORT_STRING];
+
+    snprintf (buf, sizeof (buf), "Purge %d deleted message%s?",
+	      ctx->deleted, ctx->deleted == 1 ? "" : "s");
+    if ((rc = query_quadoption (OPT_DELETE, buf)) < 0)
       return (-1);
     else if (rc == M_NO)
     {
@@ -925,12 +969,14 @@ int mx_sync_mailbox (CONTEXT *ctx)
 
     /* update memory to reflect the new state of the mailbox */
     ctx->vcount = 0;
+    ctx->vsize = 0;
     ctx->tagged = 0;
     ctx->deleted = 0;
     ctx->new = 0;
     ctx->unread = 0;
     ctx->changed = 0;
     ctx->flagged = 0;
+#define this_body ctx->hdrs[j]->content
     for (i = 0, j = 0; i < ctx->msgcount; i++)
     {
       if (!ctx->hdrs[i]->deleted)
@@ -945,6 +991,8 @@ int mx_sync_mailbox (CONTEXT *ctx)
 	{
 	  ctx->v2r[ctx->vcount] = j;
 	  ctx->hdrs[j]->virtual = ctx->vcount++;
+	  ctx->vsize += this_body->length + this_body->offset -
+	                this_body->hdr_offset;
 	}
 	ctx->hdrs[j]->changed = 0;
 	if (ctx->hdrs[j]->tagged)
@@ -961,6 +1009,10 @@ int mx_sync_mailbox (CONTEXT *ctx)
       }
       else
       {
+	if (ctx->magic == M_MH || ctx->magic == M_MAILDIR)
+	 ctx->size -= (ctx->hdrs[i]->content->length +
+	               ctx->hdrs[i]->content->offset -
+		       ctx->hdrs[i]->content->hdr_offset);
 	/* remove message from the hash tables */
 	if (ctx->hdrs[i]->env->real_subj)
 	  hash_delete (ctx->subj_hash, ctx->hdrs[i]->env->real_subj, ctx->hdrs[i], NULL);
@@ -969,6 +1021,7 @@ int mx_sync_mailbox (CONTEXT *ctx)
 	mutt_free_header (&ctx->hdrs[i]);
       }
     }
+#undef this_body
     ctx->msgcount = j;
 
     mutt_sort_headers (ctx, 1); /* rethread from scratch */
@@ -1000,7 +1053,7 @@ int mh_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
       cp = de->d_name;
       while (*cp)
       {
-	if (!isdigit (*cp))
+	if (!isdigit ((unsigned char) *cp))
 	  break;
 	cp++;
       }
@@ -1048,6 +1101,20 @@ int mbox_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
   return 0;
 }
 
+#ifdef USE_IMAP
+int imap_open_new_message (MESSAGE *msg, CONTEXT *dest, HEADER *hdr)
+{
+  char tmp[_POSIX_PATH_MAX];
+
+  mutt_mktemp(tmp);
+  if ((msg->fp = safe_fopen (tmp, "w")) == NULL)
+    return (-1);
+  msg->path = safe_strdup(tmp);
+  msg->ctx = dest;
+  return 0;
+}
+#endif
+
 /* args:
  *	dest	destintation mailbox
  *	hdr	message being copied (required for maildir support, because
@@ -1072,6 +1139,11 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
     case M_MH:
       func = mh_open_new_message;
       break;
+#ifdef USE_IMAP
+    case M_IMAP:
+      func = imap_open_new_message;
+      break;
+#endif
     default:
       dprint (1, (debugfile, "mx_open_new_message(): function unimplemented for mailbox type %d.\n",
 		  dest->magic));
@@ -1087,7 +1159,8 @@ MESSAGE *mx_open_new_message (CONTEXT *dest, HEADER *hdr, int flags)
     if (dest->magic == M_MMDF)
       fputs (MMDF_SEP, msg->fp);
 
-    if (msg->magic != M_MAILDIR && (flags & M_ADD_FROM))
+    if ((msg->magic != M_MAILDIR) && (msg->magic != M_IMAP) && 
+	(flags & M_ADD_FROM))
     {
       if (hdr)
       {
@@ -1163,6 +1236,7 @@ int mutt_reopen_mailbox (CONTEXT *ctx, int *index_hint)
   ctx->tagged = 0;
   ctx->deleted = 0;
   ctx->new = 0;
+  ctx->unread = 0;
   ctx->flagged = 0;
   ctx->changed = 0;
   ctx->id_hash = hash_create (257);
@@ -1318,9 +1392,7 @@ int mx_check_mailbox (CONTEXT *ctx, int *index_hint)
 /* return a stream pointer for a message */
 MESSAGE *mx_open_message (CONTEXT *ctx, int msgno)
 {
-  HEADER *cur = ctx->hdrs[msgno];
   MESSAGE *msg;
-  char path[_POSIX_PATH_MAX];
   
   msg = safe_calloc (1, sizeof (MESSAGE));
   switch (msg->magic = ctx->magic)
@@ -1332,14 +1404,19 @@ MESSAGE *mx_open_message (CONTEXT *ctx, int msgno)
 
     case M_MH:
     case M_MAILDIR:
-      snprintf (path, sizeof (path), "%s/%s", ctx->path, cur->path);
-      if ((msg->fp = fopen (path, "r")) == NULL)
       {
-	mutt_perror (path);
-	dprint (1, (debugfile, "mx_open_message: fopen: %s: %s (errno %d).\n",
-		    path, strerror (errno), errno));
-	free (msg);
-	msg = NULL;
+	HEADER *cur = ctx->hdrs[msgno];
+	char path[_POSIX_PATH_MAX];
+
+	snprintf (path, sizeof (path), "%s/%s", ctx->path, cur->path);
+	if ((msg->fp = fopen (path, "r")) == NULL)
+	{
+	  mutt_perror (path);
+	  dprint (1, (debugfile, "mx_open_message: fopen: %s: %s (errno %d).\n",
+		      path, strerror (errno), errno));
+	  free (msg);
+	  msg = NULL;
+	}
       }
       break;
 
@@ -1382,8 +1459,26 @@ int mx_close_message (MESSAGE **msg)
     }
   }
 
-  if ((*msg)->magic == M_MH || (*msg)->magic == M_MAILDIR || (*msg)->magic == M_IMAP)
-    r = fclose ((*msg)->fp);
+  switch ((*msg)->magic) 
+  {
+    case M_MH:
+    case M_MAILDIR:
+      r = fclose ((*msg)->fp);
+      break;
+
+#ifdef USE_IMAP
+    case M_IMAP:
+      r = fclose ((*msg)->fp);
+      if ((*msg)->write && (*msg)->ctx->append)
+      {
+	r = imap_append_message ((*msg)->ctx, *msg);
+	unlink ((*msg)->path);
+      }
+#endif
+
+    default:
+      break;
+  }
 
   free (*msg);
   *msg = NULL;
@@ -1418,6 +1513,12 @@ void mx_update_context (CONTEXT *ctx)
 {
   HEADER *h = ctx->hdrs[ctx->msgcount];
 
+
+
+
+
+
+
     if (mutt_needs_mailcap (h->content))
       h->mailcap = 1;
   if (h->flagged)
@@ -1430,7 +1531,7 @@ void mx_update_context (CONTEXT *ctx)
     if (!h->old)
       ctx->new++;
   }
-  if (ctx->vcount == ctx->msgcount)
+  if (!ctx->pattern)
   {
     ctx->v2r[ctx->vcount] = ctx->msgcount;
     h->virtual = ctx->vcount++;
