@@ -27,6 +27,15 @@
 
 
 
+
+
+
+
+
+
+
+
+
 #include <sys/stat.h>
 #include <ctype.h>
 #include <unistd.h>
@@ -1382,6 +1391,7 @@ mutt_pager (const char *banner, const char *fname, int do_color, pager_t *extra)
 	  index = mutt_new_menu();
 	  index->menu = MENU_MAIN;
 	  index->make_entry = index_make_entry;
+	  index->color = index_color;
 	  index->max = Context->vcount;
 	  index->current = extra->hdr->virtual;
 	}
@@ -1451,7 +1461,7 @@ mutt_pager (const char *banner, const char *fname, int do_color, pager_t *extra)
       {
 	_mutt_make_string (buffer,
 			   COLS-9 < sizeof (buffer) ? COLS-9 : sizeof (buffer),
-			   PagerFmt, extra->hdr, M_MAKEPRINT);
+			   NONULL (PagerFmt), Context, extra->hdr, M_FORMAT_MAKEPRINT);
       }
       printw ("%-*.*s -- (", COLS-10, COLS-10, IsHeader (extra) ? buffer : banner);
       if (last_pos < sb.st_size - 1)
@@ -1850,7 +1860,7 @@ mutt_pager (const char *banner, const char *fname, int do_color, pager_t *extra)
 
       case OP_CREATE_ALIAS:
 	CHECK_MODE(IsHeader (extra));
-	mutt_create_alias (extra->hdr->env);
+	mutt_create_alias (extra->hdr->env, NULL);
 	MAYBE_REDRAW (redraw);
 	break;
 
@@ -2137,6 +2147,8 @@ mutt_pager (const char *banner, const char *fname, int do_color, pager_t *extra)
       case OP_VIEW_ATTACHMENTS:
 	CHECK_MODE(IsHeader (extra));
 	mutt_view_attachments (extra->hdr);
+	if (extra->hdr->attach_del)
+	  Context->changed = 1;
 	redraw = REDRAW_FULL;
 	break;
 
