@@ -705,8 +705,8 @@ static time_t parse_date (char *s, HEADER *h)
 	if (*ptz == '+' || *ptz == '-')
 	{
 	  if (ptz[1] && ptz[2] && ptz[3] && ptz[4]
-	      && isdigit ((unsigned char) ptz[1]) && isdigit ((int) ptz[2])
-	      && isdigit ((unsigned char) ptz[3]) && isdigit ((int) ptz[4]))
+	      && isdigit ((unsigned char) ptz[1]) && isdigit ((unsigned char) ptz[2])
+	      && isdigit ((unsigned char) ptz[3]) && isdigit ((unsigned char) ptz[4]))
 	  {
 	    zhours = (ptz[1] - '0') * 10 + (ptz[2] - '0');
 	    zminutes = (ptz[3] - '0') * 10 + (ptz[4] - '0');
