@@ -105,7 +105,7 @@ int mutt_which_case (const char *s)
 {
   while (*s)
   {
-    if (isalpha ((unsigned char) *s) && isupper ((int) *s))
+    if (isalpha ((unsigned char) *s) && isupper ((unsigned char) *s))
       return 0; /* case-sensitive */
     s++;
   }
