@@ -59,10 +59,13 @@ static int getLine (int fd, char *s, int len)
 
 static int getPass (void)
 {
-  if (!PopPass[0])
+  if (!PopPass)
   {
-    if (mutt_get_password ("POP Password: ", PopPass, sizeof (PopPass)) != 0)
+    char tmp[SHORT_STRING];
+    tmp[0] = '\0';
+    if (mutt_get_password ("POP Password: ", tmp, sizeof (tmp)) != 0)
       return 0;
+    PopPass = safe_strdup (tmp);
   }
   return 1;
 }
@@ -82,13 +85,13 @@ void mutt_fetchPopMail (void)
   CONTEXT ctx;
   MESSAGE *msg = NULL;
 
-  if (!PopHost[0])
+  if (!PopHost)
   {
     mutt_error ("POP host is not defined.");
     return;
   }
 
-  if (!PopUser[0])
+  if (!PopUser)
   {
     mutt_error ("No POP username is defined.");
     return;
