@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -17,90 +17,93 @@
  */ 
 
 #include "mutt.h"
+#include "mime.h"
 #include "rfc2047.h"
 
 #include <ctype.h>
 #include <string.h>
 
-typedef void encode_t (char *, unsigned char *, size_t);
+typedef void encode_t (char *, size_t, const unsigned char *);
 
-extern char *tspecials;
+extern char MimeSpecials[];
 extern char B64Chars[];
 
-static void q_encode_string (char *d, unsigned char *s, size_t len)
+static void q_encode_string (char *d, size_t dlen, const unsigned char *s)
 {
   char charset[SHORT_STRING];
-  int cslen;
-  int wordlen;
+  size_t cslen, wordlen;
   char *wptr = d;
 
   snprintf (charset, sizeof (charset), "=?%s?Q?",
 	    strcasecmp ("us-ascii", charset) == 0 ? "unknown-8bit" : Charset);
   cslen = strlen (charset);
 
-  /*
-   * Hack to pull the Re: and Fwd: out of the encoded word for better
-   * handling by agents which do not support RFC2047.
-   */
-
-  if (strncasecmp ("re: ", (char *) s, 4) == 0)
-  {
-    strncpy (wptr, (char *) s, 4);
-    wptr += 4;
-    s += 4;
-  }
-  else if (strncasecmp ("fwd: ", (char *) s, 5) == 0)
-  {
-    strncpy (wptr, (char *) s, 5);
-    wptr += 5;
-    s += 5;
-  }
-
   strcpy (wptr, charset);
   wptr += cslen;
   wordlen = cslen;
+  dlen -= cslen;
 
-  while (*s)
+  dlen -= 3; /* save room for the word terminator */
+
+  while (*s && dlen > 0)
   {
     if (wordlen >= 72)
     {
-      strcpy(wptr, "?=\n ");
+      if (dlen < 4 + cslen)
+	break;
+
+      strcpy (wptr, "?=\n ");
       wptr += 4;
-      strcpy(wptr, charset);
+      dlen -= 4;
+      strcpy (wptr, charset);
       wptr += cslen;
       wordlen = cslen;
+      dlen -= cslen;
     }
 
     if (*s == ' ')
     {
       *wptr++ = '_';
       wordlen++;
+      dlen--;
     }
-    else if (*s & 0x80 || (strchr (tspecials, *s) != NULL))
+    else if ((*s & 0x80) || *s == '\t' || strchr (MimeSpecials, *s))
     {
       if (wordlen >= 70)
       {
+	if (dlen < 4 + cslen)
+	  break;
+
 	strcpy (wptr, "?=\n ");
 	wptr += 4;
+	dlen -= 4;
+
 	strcpy (wptr, charset);
 	wptr += cslen;
 	wordlen = cslen;
+	dlen -= cslen;
       }
-      sprintf (wptr, "=%2X", *s);
+
+      if (dlen < 3)
+	break;
+      sprintf (wptr, "=%02X", *s);
       wptr += 3;
       wordlen += 3;
+      dlen -= 3;
     }
     else
     {
       *wptr++ = *s;
       wordlen++;
+      dlen--;
     }
     s++;
   }
+
   strcpy (wptr, "?=");
 }
 
-static void b_encode_string (char *d, unsigned char *s, size_t len)
+static void b_encode_string (char *d, size_t dlen, const unsigned char *s)
 {
   char charset[SHORT_STRING];
   char *wptr = d;
@@ -112,16 +115,25 @@ static void b_encode_string (char *d, unsigned char *s, size_t len)
   strcpy (wptr, charset);
   wptr += cslen;
   wordlen = cslen;
+  dlen -= cslen;
 
-  while (*s)
+  dlen -= 3; /* save room for the word terminator */
+
+  while (*s && dlen >= 4)
   {
     if (wordlen >= 71)
     {
-      strcpy(wptr, "?=\n ");
+      if (dlen < 4 + cslen)
+	break;
+
+      strcpy (wptr, "?=\n ");
       wptr += 4;
-      strcpy(wptr, charset);
+      dlen -= 4;
+
+      strcpy (wptr, charset);
       wptr += cslen;
       wordlen = cslen;
+      dlen -= cslen;
     }
 
     *wptr++ = B64Chars[ (*s >> 2) & 0x3f ];
@@ -146,27 +158,33 @@ static void b_encode_string (char *d, unsigned char *s, size_t len)
     }
 
     wordlen += 4;
+    dlen -= 4;
   }
 
-  strcpy(wptr, "?=");
+  strcpy (wptr, "?=");
 }
 
-void rfc2047_encode_string (char *d, unsigned char *s, size_t l)
+void rfc2047_encode_string (char *d, size_t dlen, const unsigned char *s)
 {
   int count = 0;
   int len;
-  unsigned char *p = s;
+  const unsigned char *p = s;
   encode_t *encoder;
 
   /* First check to see if there are any 8-bit characters */
-  while (*p)
+  for (; *p; p++)
   {
-    if (*p & 0x80) count++;
-    p++;
+    if (*p & 0x80)
+      count++;
+    else if (*p == '=' && *p == '?')
+    {
+      count += 2;
+      p++;
+    }
   }
   if (!count)
   {
-    strfcpy (d, (char *)s, l);
+    strfcpy (d, (const char *)s, dlen);
     return;
   }
 
@@ -183,7 +201,24 @@ void rfc2047_encode_string (char *d, unsigned char *s, size_t l)
       encoder = b_encode_string;
   }
 
-  (*encoder)(d, s, l);
+  /* Hack to pull the Re: and Fwd: out of the encoded word for better
+     handling by agents which do not support RFC2047.  */
+  if (!strncasecmp ("re: ", (char *) s, 4))
+  {
+    strncpy (d, (char *) s, 4);
+    d += 4;
+    dlen -= 4;
+    s += 4;
+  }
+  else if (!strncasecmp ("fwd: ", (char *) s, 5))
+  {
+    strncpy (d, (char *) s, 5);
+    d += 5;
+    dlen -= 5;
+    s += 5;
+  }
+
+  (*encoder) (d, dlen, s);
 }
 
 void rfc2047_encode_adrlist (ADDRESS *addr)
@@ -195,8 +230,8 @@ void rfc2047_encode_adrlist (ADDRESS *addr)
   {
     if (ptr->personal)
     {
-      rfc2047_encode_string (buffer, (unsigned char *)ptr->personal, sizeof (buffer));
-      safe_free ((void **)&ptr->personal);
+      rfc2047_encode_string (buffer, sizeof (buffer), (const unsigned char *)ptr->personal);
+      safe_free ((void **) &ptr->personal);
       ptr->personal = safe_strdup (buffer);
     }
     ptr = ptr->next;
@@ -256,14 +291,14 @@ static int rfc2047_decode_word (char *d, const char *s, size_t len)
 	{
 	  while (*pp && len > 0)
 	  {
-	    c1 = Index_64[(int) pp[0]];
-	    c2 = Index_64[(int) pp[1]];
+	    c1 = base64val(pp[0]);
+	    c2 = base64val(pp[1]);
 	    *pd++ = (c1 << 2) | ((c2 >> 4) & 0x3);
 	    if (--len == 0) break;
 	    
 	    if (pp[2] == '=') break;
 
-	    c3 = Index_64[(int) pp[2]];
+	    c3 = base64val(pp[2]);
 	    *pd++ = ((c2 & 0xf) << 4) | ((c3 >> 2) & 0xf);
 	    if (--len == 0)
 	      break;
@@ -271,7 +306,7 @@ static int rfc2047_decode_word (char *d, const char *s, size_t len)
 	    if (pp[3] == '=')
 	      break;
 
-	    c4 = Index_64[(int) pp[3]];   
+	    c4 = base64val(pp[3]);
 	    *pd++ = ((c3 & 0x3) << 6) | c4;
 	    if (--len == 0)
 	      break;
@@ -290,7 +325,7 @@ static int rfc2047_decode_word (char *d, const char *s, size_t len)
     pd = d;
     while (*pd)
     {
-      if (!IsPrint ((unsigned char) *pd))
+      if (!IsPrint (*pd))
 	*pd = '?';
       pd++;
     }
@@ -326,7 +361,7 @@ void rfc2047_decode (char *d, const char *s, size_t dlen)
     {
       n = (size_t) (p - s);
       /* ignore spaces between encoded words */
-      if (!found_encoded || strspn (s, " \t") != n)
+      if (!found_encoded || strspn (s, " \t\r\n") != n)
       {
 	if (n > dlen)
 	  n = dlen;
